@@ -1,12 +1,12 @@
 #! /bin/sh
 
-if [ ! -x $(which pkg-config) ]; then
+if ! pkg-config --version > /dev/null 2>&1 ; then
   echo 'pkg-config not found.' >&2
   echo 'Make sure that pkg-config is installed on your system.' >&2
   exit 1
 fi
 
-if [ -x "`which autoreconf 2>/dev/null`" ] ; then
+if autoreconf --version > /dev/null 2>&1 ; then
   exec autoreconf -ivf
 fi
 
