@@ -14,17 +14,23 @@ if [ ! -d "$SODIUM_PREFIX" ]; then
   sleep 10
 fi
 
-./configure --prefix="$PREFIX" --exec-prefix="$PREFIX" \
+./configure \
   --host=i686-w64-mingw32 \
+  --with-included-ltdl \
   --bindir="$PREFIX" \
+  --datarootdir="$PREFIX" \
+  --exec-prefix="$PREFIX" \
+  --prefix="$PREFIX" \
   --sbindir="$PREFIX" \
-  --with-included-ltdl && \
+  --sysconfdir="$PREFIX" && \
 make install
 
+rm -fr "${PREFIX}/man"
 rm -fr "${PREFIX}/share"
 rm -fr "${PREFIX}/lib/pkgconfig"
 mv "${PREFIX}/lib/dnscrypt-proxy/"*.dll "${PREFIX}/"
-mv "${PREFIX}/lib/dnscrypt-proxy/"*.la "${PREFIX}/"
+mv "${PREFIX}/dnscrypt-proxy/"* "${PREFIX}/"
+rmdir "${PREFIX}/dnscrypt-proxy"
 rm -fr "${PREFIX}/lib"
 cp "${MINGW_PREFIX}/bin/libwinpthread-1.dll" "${PREFIX}/"
 cp "${MINGW_PREFIX}/bin/libgcc_s_dw2-1.dll" "${PREFIX}/"
@@ -34,5 +40,3 @@ cp "${SODIUM_PREFIX}/bin/libsodium-18.dll" "${PREFIX}/"
 nm "${MINGW_PREFIX}/bin/libldns-1.dll" | \
   fgrep -i libeay > /dev/null &&
   cp "${MINGW_PREFIX}/bin/"libeay*.dll "${PREFIX}/"
-
-cp dnscrypt-resolvers.csv "${PREFIX}/"
