@@ -25,6 +25,7 @@
 #include "dnscrypt_proxy.h"
 #include "logger.h"
 #include "probes.h"
+#include "shims.h"
 #include "utils.h"
 
 static int cert_updater_update(ProxyContext * const proxy_context);
@@ -34,8 +35,8 @@ cert_parse_version(ProxyContext * const proxy_context,
                    const SignedBincert * const signed_bincert,
                    const size_t signed_bincert_len)
 {
-    if (signed_bincert_len <= (size_t) (signed_bincert->signed_data -
-                                        signed_bincert->magic_cert) ||
+    if (signed_bincert_len < (size_t) (signed_bincert->signed_data -
+                                       signed_bincert->magic_cert) ||
         memcmp(signed_bincert->magic_cert, CERT_MAGIC_CERT,
                sizeof signed_bincert->magic_cert) != 0) {
         logger_noformat(proxy_context, LOG_DEBUG,
@@ -43,9 +44,15 @@ cert_parse_version(ProxyContext * const proxy_context,
         return -1;
     }
     if (signed_bincert->version_major[0] != 0U ||
-        signed_bincert->version_major[1] != 1U) {
-        logger_noformat(proxy_context, LOG_WARNING,
-                        "Unsupported certificate version");
+        (signed_bincert->version_major[1] != 1U
+#ifdef HAVE_XCHACHA20
+         && signed_bincert->version_major[1] != 2U
+#endif
+        )) {
+        logger(proxy_context, LOG_INFO,
+               "Unsupported certificate version: %u.%u",
+               signed_bincert->version_major[1],
+               signed_bincert->version_major[0]);
         return -1;
     }
     return 0;
@@ -59,9 +66,15 @@ cert_parse_bincert(ProxyContext * const proxy_context,
     uint32_t serial;
     memcpy(&serial, bincert->serial, sizeof serial);
     serial = htonl(serial);
-    logger(proxy_context, LOG_INFO,
-           "Server certificate #%" PRIu32 " received", serial);
-
+    if (serial >= 0x30303030 && serial <= 0x30303039) {
+        logger(proxy_context, LOG_INFO,
+               "Server certificate with serial '%c%c%c%c' received",
+               (serial >> 24) & 0xff, (serial >> 16) & 0xff,
+               (serial >> 8) & 0xff, serial & 0xff);
+    } else {
+        logger(proxy_context, LOG_INFO,
+               "Server certificate with serial #%" PRIu32 " received", serial);
+    }
     uint32_t ts_begin;
     memcpy(&ts_begin, bincert->ts_begin, sizeof ts_begin);
     ts_begin = htonl(ts_begin);
@@ -70,12 +83,24 @@ cert_parse_bincert(ProxyContext * const proxy_context,
     memcpy(&ts_end, bincert->ts_end, sizeof ts_end);
     ts_end = htonl(ts_end);
 
+    if (ts_end <= ts_begin) {
+        logger_noformat(proxy_context, LOG_WARNING,
+                        "This certificate has a bogus validity period");
+        return -1;
+    }
+
     const uint32_t now_u32 = (uint32_t) time(NULL);
 
     if (now_u32 < ts_begin) {
-        logger_noformat(proxy_context, LOG_INFO,
-                        "This certificate has not been activated yet");
-        return -1;
+        if (proxy_context->ignore_timestamps != 0) {
+            logger_noformat(proxy_context, LOG_WARNING,
+                            "Clock might be off - "
+                            "Pretending that this certificate is valid no matter what");
+        } else {
+            logger_noformat(proxy_context, LOG_INFO,
+                            "This certificate has not been activated yet");
+            return -1;
+        }
     }
     if (now_u32 > ts_end) {
         logger_noformat(proxy_context, LOG_INFO,
@@ -87,9 +112,33 @@ cert_parse_bincert(ProxyContext * const proxy_context,
         return 0;
     }
 
+    const uint32_t version =
+        (((uint32_t) bincert->version_major[0]) << 24) +
+        (((uint32_t) bincert->version_major[1]) << 16) +
+        (((uint32_t) bincert->version_minor[0]) << 8) +
+        (((uint32_t) bincert->version_minor[1]));
+
+    const uint32_t previous_version =
+        (((uint32_t) previous_bincert->version_major[0]) << 24) +
+        (((uint32_t) previous_bincert->version_major[1]) << 16) +
+        (((uint32_t) previous_bincert->version_minor[0]) << 8) +
+        (((uint32_t) previous_bincert->version_minor[1]));
+
     uint32_t previous_serial;
     memcpy(&previous_serial, previous_bincert->serial, sizeof previous_serial);
     previous_serial = htonl(previous_serial);
+
+    if (previous_version > version) {
+        logger(proxy_context, LOG_INFO, "Keeping certificate #%" PRIu32 " "
+               "which is for a more recent version than #%" PRIu32,
+               previous_serial, serial);
+        return -1;
+    } else if (previous_version < version) {
+        logger(proxy_context, LOG_INFO,
+               "Favoring version #%" PRIu32 " over version #%" PRIu32,
+               version, previous_version);
+        return 0;
+    }
     if (previous_serial > serial) {
         logger(proxy_context, LOG_INFO, "Certificate #%" PRIu32 " "
                "has been superseded by certificate #%" PRIu32,
@@ -124,12 +173,13 @@ cert_open_bincert(ProxyContext * const proxy_context,
         DNSCRYPT_PROXY_CERTS_UPDATE_ERROR_COMMUNICATION();
         return -1;
     }
-    assert(signed_bincert_len >= (size_t) (signed_bincert->signed_data -
-                                           signed_bincert->magic_cert));
+    assert(signed_bincert_len > (size_t) (signed_bincert->signed_data -
+                                          signed_bincert->magic_cert));
     signed_data_len = signed_bincert_len -
         (size_t) (signed_bincert->signed_data - signed_bincert->magic_cert);
     assert(bincert_size - (size_t) (bincert->server_publickey -
                                     bincert->magic_cert) == signed_data_len);
+    memcpy(bincert, signed_bincert, signed_bincert_len - signed_data_len);
     if (crypto_sign_ed25519_open(bincert->server_publickey, &bincert_data_len_ul,
                                  signed_bincert->signed_data, signed_data_len,
                                  proxy_context->provider_publickey) != 0) {
@@ -140,12 +190,12 @@ cert_open_bincert(ProxyContext * const proxy_context,
         return -1;
     }
     if (cert_parse_bincert(proxy_context, bincert, *bincert_p) != 0) {
-        memset(bincert, 0, sizeof *bincert);
+        sodium_memzero(bincert, sizeof *bincert);
         free(bincert);
         return -1;
     }
     if (*bincert_p != NULL) {
-        memset(*bincert_p, 0, sizeof **bincert_p);
+        sodium_memzero(*bincert_p, sizeof **bincert_p);
         free(*bincert_p);
     }
     *bincert_p = bincert;
@@ -157,9 +207,6 @@ static void
 cert_print_bincert_info(ProxyContext * const proxy_context,
                         const Bincert * const bincert)
 {
-    (void) proxy_context;
-    (void) bincert;
-
 #ifdef HAVE_GMTIME_R
     struct tm ts_begin_tm;
     struct tm ts_end_tm;
@@ -192,10 +239,17 @@ cert_print_bincert_info(ProxyContext * const proxy_context,
            "from [%d-%02d-%02d] to [%d-%02d-%02d]",
            htonl(serial),
            ts_begin_tm.tm_year + 1900,
-           ts_begin_tm.tm_mon + 1, ts_begin_tm.tm_mday + 1,
+           ts_begin_tm.tm_mon + 1, ts_begin_tm.tm_mday,
            ts_end_tm.tm_year + 1900,
-           ts_end_tm.tm_mon + 1, ts_end_tm.tm_mday + 1);
+           ts_end_tm.tm_mon + 1, ts_end_tm.tm_mday);
 #endif
+    if (bincert->version_major[1] > 1U) {
+        const unsigned int version_minor =
+            bincert->version_minor[0] * 256 + bincert->version_minor[1];
+        logger(proxy_context, LOG_INFO,
+               "Using version %u.%u of the DNSCrypt protocol",
+               bincert->version_major[1], version_minor);
+    }
 }
 
 static void
@@ -272,6 +326,25 @@ cert_reschedule_query_after_success(ProxyContext * const proxy_context)
                           (CERT_QUERY_RETRY_DELAY_AFTER_SUCCESS_JITTER));
 }
 
+static void
+cert_check_key_rotation_period(ProxyContext * const proxy_context,
+                               const Bincert * const bincert)
+{
+    uint32_t ts_begin;
+    uint32_t ts_end;
+
+    memcpy(&ts_begin, bincert->ts_begin, sizeof ts_begin);
+    ts_begin = htonl(ts_begin);
+    memcpy(&ts_end, bincert->ts_end, sizeof ts_end);
+    ts_end = htonl(ts_end);
+    assert(ts_end > ts_begin);
+    if (ts_end - ts_begin > CERT_RECOMMENDED_MAX_KEY_ROTATION_PERIOD) {
+        logger_noformat(proxy_context, LOG_INFO,
+                        "The key rotation period for this server may exceed the recommended value. "
+                        "This is bad for forward secrecy.");
+    }
+}
+
 static void
 cert_query_cb(int result, char type, int count, int ttl,
               void * const txt_records_, void * const arg)
@@ -279,6 +352,7 @@ cert_query_cb(int result, char type, int count, int ttl,
     Bincert                 *bincert = NULL;
     ProxyContext            *proxy_context = arg;
     const struct txt_record *txt_records = txt_records_;
+    Cipher                   cipher = CIPHER_UNDEFINED;
     int                      i = 0;
 
     (void) type;
@@ -303,24 +377,53 @@ cert_query_cb(int result, char type, int count, int ttl,
     if (bincert == NULL) {
         logger_noformat(proxy_context, LOG_ERR,
                         "No useable certificates found");
-        cert_reschedule_query_after_failure(proxy_context);
         DNSCRYPT_PROXY_CERTS_UPDATE_ERROR_NOCERTS();
         if (proxy_context->test_only) {
             exit(DNSCRYPT_EXIT_CERT_NOCERTS);
         }
+        cert_reschedule_query_after_failure(proxy_context);
+        return;
+    }
+    switch (bincert->version_major[1]) {
+    case 1:
+        cipher = CIPHER_XSALSA20POLY1305;
+        break;
+#ifdef HAVE_XCHACHA20
+    case 2:
+        cipher = CIPHER_XCHACHA20POLY1305;
+        break;
+#endif
+    default:
+        logger_noformat(proxy_context, LOG_ERR,
+                        "Unsupported certificate version");
+        cert_reschedule_query_after_failure(proxy_context);
         return;
     }
     if (proxy_context->test_only != 0) {
         const uint32_t now_u32 = (uint32_t) time(NULL);
+        uint32_t       ts_begin;
         uint32_t       ts_end;
+        uint32_t       safe_end;
 
+        memcpy(&ts_begin, bincert->ts_begin, sizeof ts_begin);
+        ts_begin = htonl(ts_begin);
         memcpy(&ts_end, bincert->ts_end, sizeof ts_end);
-        ts_end = htonl(ts_end);
-
-        if (ts_end < (uint32_t) proxy_context->test_cert_margin ||
-            now_u32 > ts_end - (uint32_t) proxy_context->test_cert_margin) {
+        safe_end = ts_end = htonl(ts_end);
+        if (safe_end > (uint32_t) proxy_context->test_cert_margin) {
+            safe_end -= (uint32_t) proxy_context->test_cert_margin;
+        } else {
+            safe_end = ts_begin;
+        }
+        if (safe_end < ts_begin) {
             logger_noformat(proxy_context, LOG_WARNING,
-                            "The certificate is not valid for the given safety margin");
+                            "Safety margin wider than the certificate validity period");
+            safe_end = ts_begin;
+        }
+        if (now_u32 < ts_begin || now_u32 > safe_end) {
+            logger(proxy_context, LOG_WARNING,
+                   "The certificate is not valid for the given safety margin (%lu-%lu not within [%lu..%lu])",
+                   (unsigned long) now_u32, (unsigned long) proxy_context->test_cert_margin,
+                   (unsigned long) ts_begin, (unsigned long) safe_end);
             DNSCRYPT_PROXY_CERTS_UPDATE_ERROR_NOCERTS();
             exit(DNSCRYPT_EXIT_CERT_MARGIN);
         }
@@ -334,10 +437,11 @@ cert_query_cb(int result, char type, int count, int ttl,
     memcpy(proxy_context->dnscrypt_magic_query, bincert->magic_query,
            sizeof proxy_context->dnscrypt_magic_query);
     cert_print_bincert_info(proxy_context, bincert);
+    cert_check_key_rotation_period(proxy_context, bincert);
     cert_print_server_key(proxy_context);
     dnscrypt_client_init_magic_query(&proxy_context->dnscrypt_client,
-                                     bincert->magic_query);
-    memset(bincert, 0, sizeof *bincert);
+                                     bincert->magic_query, cipher);
+    sodium_memzero(bincert, sizeof *bincert);
     free(bincert);
     if (proxy_context->test_only) {
         DNSCRYPT_PROXY_CERTS_UPDATE_DONE((unsigned char *)
@@ -396,8 +500,9 @@ cert_updater_update(ProxyContext * const proxy_context)
         return -1;
     }
     if (proxy_context->tcp_only != 0) {
-        (void) evdns_base_nameserver_ip_add(cert_updater->evdns_base,
-                                            proxy_context->resolver_ip);
+        (void) evdns_base_set_option(cert_updater->evdns_base, "use-tcp", "always");
+    } else {
+        (void) evdns_base_set_option(cert_updater->evdns_base, "use-tcp", "on-tc");
     }
     if (evdns_base_resolve_txt(cert_updater->evdns_base,
                                proxy_context->provider_name,
