@@ -48,10 +48,6 @@
 # endif
 #endif
 
-#ifndef DEFAULT_RESOLVER_NAME
-# define DEFAULT_RESOLVER_NAME NULL
-#endif
-
 #define DNS_HEADER_SIZE  12U
 #define DNS_FLAGS_TC      2U
 #define DNS_FLAGS_QR    128U
