@@ -10,7 +10,7 @@
 #include "dnscrypt_proxy.h"
 #include "edns.h"
 
-#define DNS_MAX_HOSTNAME_LEN 255U
+#define DNS_MAX_HOSTNAME_LEN 256U
 
 static int
 _skip_name(const uint8_t * const dns_packet, const size_t dns_packet_len,
@@ -18,28 +18,32 @@ _skip_name(const uint8_t * const dns_packet, const size_t dns_packet_len,
 {
     size_t  offset = *offset_p;
     size_t  name_len = (size_t) 0U;
-    uint8_t name_component_len;
+    uint8_t label_len;
 
     if (dns_packet_len < (size_t) 1U ||
         offset >= dns_packet_len - (size_t) 1U) {
         return -1;
     }
     for (;;) {
-        name_component_len = dns_packet[offset];
-        if (name_component_len == 0U) {
+        label_len = dns_packet[offset];
+        if ((label_len & 0xC0) == 0xC0) {
+            if (2U > dns_packet_len - offset) {
+                return -1;
+            }
+            offset += 2U;
             break;
         }
-        if ((name_component_len & 0xC0) == 0xC0) {
-            name_component_len = 1U;
-        }
-        if (name_component_len >= dns_packet_len - offset - 1U) {
+        if (label_len >= dns_packet_len - offset - 1U) {
             return -1;
         }
-        name_len += (size_t) name_component_len + (size_t) 1U;
+        name_len += (size_t) label_len + (size_t) 1U;
         if (name_len > DNS_MAX_HOSTNAME_LEN) {
             return -1;
         }
-        offset += name_component_len + 1U;
+        offset += label_len + 1U;
+        if (label_len == 0U) {
+            break;
+        }
     }
     if (offset >= dns_packet_len) {
         return -1;
