@@ -9,6 +9,13 @@
 #ifndef TCP_REQUEST_BACKLOG
 # define TCP_REQUEST_BACKLOG 128
 #endif
+#ifndef TCP_FASTOPEN_QUEUES
+# ifdef __APPLE__
+#  define TCP_FASTOPEN_QUEUES 1
+# else
+#  define TCP_FASTOPEN_QUEUES 5
+# endif
+#endif
 
 int tcp_listener_bind(ProxyContext * const proxy_context);
 int tcp_listener_start(ProxyContext * const proxy_context);
