@@ -1,9 +1,12 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include "app.h"
 #include "dnscrypt_proxy.h"
 #include "logger.h"
+#include "utils.h"
 #include "windows_service.h"
 
 #ifndef _WIN32
@@ -28,6 +31,7 @@ main(int argc, char *argv[])
 #include "logger.h"
 #include "utils.h"
 
+static char                 *windows_service_name;
 static SERVICE_STATUS        service_status;
 static SERVICE_STATUS_HANDLE service_status_handle;
 static _Bool                 app_is_running_as_a_service;
@@ -124,6 +128,29 @@ windows_service_parse_multi_sz(WindowsServiceParseMultiSzCb * const cb,
     return 0;
 }
 
+char *
+windows_service_registry_parameters_key(void)
+{
+    static unsigned char *key;
+    size_t                sizeof_key;
+
+    if (key != NULL) {
+        return key;
+    }
+    sizeof_key = (sizeof WINDOWS_SERVICE_REGISTRY_PARAMETERS_PATH - 1) +
+        strlen(get_windows_service_name()) +
+        (sizeof WINDOWS_SERVICE_REGISTRY_PARAMETERS_NAME - 1) + 1;
+    if ((key = malloc(sizeof_key)) == NULL) {
+        exit(1);
+    }
+    evutil_snprintf(key, sizeof_key, "%s%s%s",
+                    WINDOWS_SERVICE_REGISTRY_PARAMETERS_PATH,
+                    get_windows_service_name(),
+                    WINDOWS_SERVICE_REGISTRY_PARAMETERS_NAME);
+
+    return key;
+}
+
 static int
 windows_service_registry_read_multi_sz(const char * const key,
                                        WindowsServiceParseMultiSzCb * const cb)
@@ -134,7 +161,7 @@ windows_service_registry_read_multi_sz(const char * const key,
     DWORD   value_type;
 
     if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                     WINDOWS_SERVICE_REGISTRY_PARAMETERS_KEY,
+                     windows_service_registry_parameters_key(),
                      (DWORD) 0, KEY_READ, &hk) != ERROR_SUCCESS) {
         return -1;
     }
@@ -174,7 +201,7 @@ windows_service_registry_read_string(const char * const key,
 
     *value_p = NULL;
     if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                     WINDOWS_SERVICE_REGISTRY_PARAMETERS_KEY,
+                     windows_service_registry_parameters_key(),
                      (DWORD) 0, KEY_READ, &hk) != ERROR_SUCCESS) {
         return -1;
     }
@@ -210,7 +237,7 @@ windows_service_registry_read_dword(const char * const key,
 
     *value_p = (DWORD) 0;
     if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
-                     WINDOWS_SERVICE_REGISTRY_PARAMETERS_KEY,
+                     windows_service_registry_parameters_key(),
                      (DWORD) 0, KEY_READ, &hk) != ERROR_SUCCESS) {
         return -1;
     }
@@ -237,7 +264,7 @@ windows_service_registry_write_string(const char * const key,
         return -1;
     }
     if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,
-                       WINDOWS_SERVICE_REGISTRY_PARAMETERS_KEY,
+                       windows_service_registry_parameters_key(),
                        (DWORD) 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE,
                        NULL, &hk, NULL) != ERROR_SUCCESS) {
         return -1;
@@ -263,6 +290,12 @@ windows_build_command_line_from_registry(int * const argc_p,
     if ((*argv_p = cmdline_clone_options(*argc_p, *argv_p)) == NULL) {
         exit(1);
     }
+    if (windows_service_registry_read_string
+        ("ConfigFile", &string_value) == 0) {
+        err += cmdline_add_option(argc_p, argv_p, string_value);
+        free(string_value);
+        return -(err != 0);
+    }
     if (windows_service_registry_read_string
         ("ResolversList", &string_value) == 0) {
         err += cmdline_add_option(argc_p, argv_p, "--resolvers-list");
@@ -321,6 +354,10 @@ windows_build_command_line_from_registry(int * const argc_p,
         ("EphemeralKeys", &dword_value) == 0 && dword_value > (DWORD) 0) {
         err += cmdline_add_option(argc_p, argv_p, "--ephemeral-keys");
     }
+    if (windows_service_registry_read_dword
+        ("IgnoreTimestamps", &dword_value) == 0 && dword_value > (DWORD) 0) {
+        err += cmdline_add_option(argc_p, argv_p, "--ignore-timestamps");
+    }
     if (windows_service_registry_read_string
         ("ClientKeyFile", &string_value) == 0) {
         err += cmdline_add_option(argc_p, argv_p, "--client-key");
@@ -333,6 +370,13 @@ windows_build_command_line_from_registry(int * const argc_p,
         err += cmdline_add_option(argc_p, argv_p, string_value);
         free(string_value);
     }
+    if (windows_service_registry_read_dword
+        ("LogLevel", &dword_value) == 0) {
+        evutil_snprintf(dword_string, sizeof dword_string, "%ld",
+                        (long) dword_value);
+        err += cmdline_add_option(argc_p, argv_p, "--loglevel");
+        err += cmdline_add_option(argc_p, argv_p, dword_string);
+    }
     windows_service_registry_read_multi_sz
         ("Plugins", & (WindowsServiceParseMultiSzCb) {
             .cb = windows_service_parse_multi_sz_cb,
@@ -346,12 +390,62 @@ windows_build_command_line_from_registry(int * const argc_p,
     return 0;
 }
 
+char *
+get_windows_service_name(void)
+{
+    assert(windows_service_name != NULL);
+
+    return windows_service_name;
+}
+
+static void
+set_windows_service_name(char *name)
+{
+    windows_service_name = name;
+}
+
+static int
+init_windows_service_name(int argc, char *argv[])
+{
+    int i;
+
+    for (i = 0; i < argc; i++) {
+        if (strcmp(argv[i], "--service-name") == 0) {
+            if (i < argc - 1) {
+                windows_service_name = argv[i + 1];
+                return 0;
+            } else {
+                logger_noformat(NULL, LOG_ERR,
+                                "Missing service name after --service-name");
+                exit(1);
+            }
+        }
+        if (strncmp(argv[i], "--service-name=", sizeof "--service-name=" - 1) == 0) {
+            windows_service_name = argv[i] + sizeof "--service-name=" - 1;
+            if (*windows_service_name != 0) {
+                return 0;
+            } else {
+                logger_noformat(NULL, LOG_ERR,
+                                "Missing service name after --service-name");
+                exit(1);
+            }
+        }
+    }
+    if ((windows_service_name = strdup(WINDOWS_SERVICE_NAME)) == NULL) {
+        return -1;
+    }
+    return 0;
+}
+
 static void WINAPI
 service_main(DWORD argc_, LPTSTR *argv_)
 {
     char **argv = (char **) argv_;
     int    argc = (int) argc_;
 
+    if (argc != 0) {
+        set_windows_service_name(argv[0]);
+    }
     assert(argc_ < INT_MAX);
     if (windows_build_command_line_from_registry(&argc, &argv) != 0) {
         logger_noformat(NULL, LOG_ERR,
@@ -363,7 +457,7 @@ service_main(DWORD argc_, LPTSTR *argv_)
     service_status.dwCurrentState = SERVICE_START_PENDING;
     service_status.dwControlsAccepted = SERVICE_ACCEPT_STOP;
     service_status_handle =
-        RegisterServiceCtrlHandler(WINDOWS_SERVICE_NAME, control_handler);
+        RegisterServiceCtrlHandler(get_windows_service_name(), control_handler);
     if (service_status_handle == 0) {
         return;
     }
@@ -377,18 +471,15 @@ static int
 windows_main(int argc, char *argv[])
 {
     static SERVICE_TABLE_ENTRY service_table[2];
-    char                      *service_name;
 
-    if ((service_name = strdup(WINDOWS_SERVICE_NAME)) == NULL) {
-        perror("strdup");
-        return 1;
+    if (init_windows_service_name(argc, argv) != 0) {
+        return -1;
     }
     memcpy(service_table, (SERVICE_TABLE_ENTRY[2]) {
-        { .lpServiceName = service_name, .lpServiceProc = service_main },
-        { .lpServiceName = NULL,         .lpServiceProc = (void *) NULL }
+        { .lpServiceName = get_windows_service_name(), .lpServiceProc = service_main },
+        { .lpServiceName = NULL, .lpServiceProc = (void *) NULL }
     }, sizeof service_table);
     if (StartServiceCtrlDispatcher(service_table) == 0) {
-        free(service_name);
         return dnscrypt_proxy_main(argc, argv);
     }
     app_is_running_as_a_service = 1;
@@ -408,7 +499,7 @@ windows_service_uninstall(void)
     if (scm_handle == NULL) {
         return -1;
     }
-    service_handle = OpenService(scm_handle, WINDOWS_SERVICE_NAME,
+    service_handle = OpenService(scm_handle, get_windows_service_name(),
                                  DELETE | SERVICE_STOP);
     if (service_handle == NULL) {
         CloseServiceHandle(scm_handle);
@@ -424,6 +515,22 @@ windows_service_uninstall(void)
     return ret;
 }
 
+static int
+windows_registry_install_with_config_file(const char *config_file)
+{
+    char *config_file_path;
+    int   ret;
+
+    if ((config_file_path = path_from_app_folder(config_file)) == NULL) {
+        return -1;
+    }
+    ret = windows_service_registry_write_string("ConfigFile",
+                                                config_file_path);
+    free(config_file_path);
+
+    return ret;
+}
+
 static int
 windows_registry_install(ProxyContext * const proxy_context)
 {
@@ -450,19 +557,14 @@ windows_registry_install(ProxyContext * const proxy_context)
     return 0;
 }
 
-int
-windows_service_install(ProxyContext * const proxy_context)
+static int
+register_service(void)
 {
     char      self_path[MAX_PATH];
     char      self_path_quoted[2U + MAX_PATH];
     SC_HANDLE scm_handle;
     SC_HANDLE service_handle;
 
-    if (windows_registry_install(proxy_context) != 0) {
-        logger_noformat(proxy_context, LOG_ERR,
-                        "Unable to set up registry keys");
-        return -1;
-    }
     if (GetModuleFileName(NULL, self_path, MAX_PATH) <= (DWORD) 0) {
         return -1;
     }
@@ -476,8 +578,8 @@ windows_service_install(ProxyContext * const proxy_context)
         return -1;
     }
     service_handle = CreateService
-        (scm_handle, WINDOWS_SERVICE_NAME,
-         WINDOWS_SERVICE_NAME, SERVICE_ALL_ACCESS,
+        (scm_handle, get_windows_service_name(),
+         get_windows_service_name(), SERVICE_ALL_ACCESS,
          SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START,
          SERVICE_ERROR_NORMAL, self_path_quoted, NULL, NULL, NULL, NULL, NULL);
     if (service_handle == NULL) {
@@ -491,6 +593,29 @@ windows_service_install(ProxyContext * const proxy_context)
     return 0;
 }
 
+int
+windows_service_install(ProxyContext * const proxy_context)
+{
+    if (windows_registry_install(proxy_context) != 0) {
+        logger_noformat(proxy_context, LOG_ERR,
+                        "Unable to set up registry keys");
+        return -1;
+    }
+    return register_service();
+}
+
+int
+windows_service_install_with_config_file(ProxyContext * const proxy_context,
+                                         const char * const config_file)
+{
+    if (windows_registry_install_with_config_file(config_file) != 0) {
+        logger_noformat(proxy_context, LOG_ERR,
+                        "Unable to set up registry keys for the config file");
+        return -1;
+    }
+    return register_service();
+}
+
 int
 main(int argc, char *argv[])
 {
